import { Component, Element, Event, Method, Prop, h, Host } from '@stencil/core';
import { hideLazyLoadImages, afterSwipe, lazyLoadContent, hideAllRevealElements, showAllRevealElements } from '@deckdeckgo/slide-utils';
const capitalize = (str) => str.charAt(0).toUpperCase() + str.slice(1);
export class DeckdeckgoSlideBigImg {
  constructor() {
    this.imgSrc = '';
    this.imgDivisions = '';
    this.axis = 'x';
    this.reverse = false;
    this.currentStep = -1;
  }
  get divisions() {
    return this.imgDivisions.split(';').map((str) => {
      const num = parseInt(str);
      if (isNaN(num)) {
        return 0;
      }
      return num;
    });
  }
  async componentDidLoad() {
    await hideLazyLoadImages(this.el);
    this.crop = this.el.shadowRoot.querySelector('.deckgo-big-img-container');
    this.bigImg = this.el.shadowRoot.querySelector('img');
    this.slideDidLoad.emit();
  }
  next() {
    return new Promise((resolve) => {
      this.prevNext(true);
      resolve();
    });
  }
  prev() {
    return new Promise((resolve) => {
      this.prevNext(false);
      resolve();
    });
  }
  prevNext(next) {
    const axisDimension = this.axis === 'x' ? 'width' : 'height';
    const perpendicularAxisDimension = this.axis === 'y' ? 'width' : 'height';
    const axisMarginStart = `margin${this.axis === 'x' ? 'Left' : 'Top'}`;
    if (this.currentStep === -1 && next) {
      this.currentStep = this.reverse ? this.divisions.length : 0;
    }
    else if (this.currentStep === this.divisions.length && !next && this.reverse) {
      this.currentStep = -1;
    }
    else {
      this.currentStep = this.currentStep + (this.reverse ? -1 : 1) * (next ? 1 : -1);
    }
    if (this.currentStep === -1) {
      this.bigImg.classList.remove('cropped');
      this.bigImg.style[axisMarginStart] = '';
      this.bigImg.style[perpendicularAxisDimension] = '';
      this.crop.style[axisDimension] = '';
      this.crop.style[perpendicularAxisDimension] = '';
    }
    else {
      this.crop.style[perpendicularAxisDimension] = '100%';
      this.bigImg.style[perpendicularAxisDimension] = '100%';
      this.bigImg.classList.add('cropped');
      const previousNaturalDivision = this.currentStep === 0 ? 0 : this.divisions[this.currentStep - 1];
      const calcCrop = () => {
        const imgClientLength = this.bigImg[`client${capitalize(axisDimension)}`];
        const imgNaturalLength = this.bigImg[`natural${capitalize(axisDimension)}`];
        const lengthFactor = imgClientLength / imgNaturalLength;
        const currentNaturalDivision = this.currentStep === this.divisions.length ? imgNaturalLength : this.divisions[this.currentStep];
        return {
          length: (currentNaturalDivision - previousNaturalDivision) * lengthFactor,
          shift: -(previousNaturalDivision * lengthFactor),
        };
      };
      let crop = calcCrop();
      if (crop.length > this.el.shadowRoot.querySelector('.deckgo-big-img-container').clientHeight) {
        this.crop.style[perpendicularAxisDimension] = '';
        crop = calcCrop();
      }
      this.crop.style[axisDimension] = crop.length + 'px';
      this.bigImg.style[axisMarginStart] = crop.shift + 'px';
    }
  }
  isEnd() {
    return this.reverse ? this.currentStep === 0 : this.currentStep === this.divisions.length;
  }
  isBeginning() {
    return this.currentStep === -1;
  }
  beforeSwipe(enter) {
    return new Promise(async (resolve) => {
      const couldSwipe = !this.divisions[0] || (enter ? this.isEnd() : this.isBeginning());
      if (couldSwipe) {
        resolve(true);
        return;
      }
      if (enter) {
        await this.next();
      }
      else {
        await this.prev();
      }
      resolve(false);
    });
  }
  afterSwipe() {
    return afterSwipe();
  }
  lazyLoadContent() {
    return lazyLoadContent(this.el);
  }
  revealContent() {
    return showAllRevealElements(this.el);
  }
  hideContent() {
    return hideAllRevealElements(this.el);
  }
  render() {
    return (h(Host, { class: { 'deckgo-slide-container': true } },
      h("div", { class: "deckgo-slide" },
        h("slot", { name: "title" }),
        h("div", { class: "deckgo-big-img-container" },
          h("img", { "data-src": this.imgSrc, alt: this.imgAlt }),
          h("slot", { name: "notes" }),
          h("slot", { name: "actions" }),
          h("slot", { name: "background" }),
          h("slot", { name: "header" }),
          h("slot", { name: "footer" })))));
  }
  static get is() { return "deckgo-slide-big-img"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["deckdeckgo-slide-big-img.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["deckdeckgo-slide-big-img.css"]
  }; }
  static get properties() { return {
    "imgSrc": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "img-src",
      "reflect": false,
      "defaultValue": "''"
    },
    "imgAlt": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "img-alt",
      "reflect": false
    },
    "imgDivisions": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "img-divisions",
      "reflect": false,
      "defaultValue": "''"
    },
    "axis": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'x' | 'y'",
        "resolved": "\"x\" | \"y\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "axis",
      "reflect": false,
      "defaultValue": "'x'"
    },
    "reverse": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "reverse",
      "reflect": false,
      "defaultValue": "false"
    }
  }; }
  static get events() { return [{
      "method": "slideDidLoad",
      "name": "slideDidLoad",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "beforeSwipe": {
      "complexType": {
        "signature": "(enter: boolean) => Promise<boolean>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<boolean>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "afterSwipe": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "lazyLoadContent": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "revealContent": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "hideContent": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
}
