import { r as registerInstance, c as createEvent, h, H as Host, g as getElement } from './index-3552aa55.js';

function getAllElements(el, tag) {
    const allSlottedElements = el.querySelectorAll(tag);
    const allShadowsElements = el.shadowRoot ? el.shadowRoot.querySelectorAll(tag) : [];
    return Array.from(allSlottedElements).concat(Array.from(allShadowsElements));
}

function lazyLoadComponentContent(el, tag) {
    return new Promise(async (resolve) => {
        const promises = [];
        const elements = getAllElements(el, tag);
        if (elements && elements.length > 0) {
            elements.forEach((element) => {
                promises.push(element.lazyLoadContent());
            });
            await Promise.all(promises);
            resolve();
        }
        resolve();
    });
}

function lazyLoadSelectedImages(images) {
    return new Promise((resolve) => {
        if (!images) {
            resolve();
            return;
        }
        images.forEach((image) => {
            if (image.hasAttribute('data-src')) {
                image.setAttribute('src', `${image.getAttribute('data-src')}`);
                image.removeAttribute('data-src');
                if (!image.classList.contains('deckgo-reveal')) {
                    image.style.setProperty('visibility', 'inherit');
                }
            }
            image.style.setProperty('pointer-events', 'none');
        });
        resolve();
    });
}
function lazyLoadSelectedLazyImagesComponent(components) {
    return new Promise((resolve) => {
        if (!components) {
            resolve();
            return;
        }
        components.forEach(async (component) => {
            await component.lazyLoad();
        });
        resolve();
    });
}

function lazyLoadImages(el) {
    return new Promise(async (resolve) => {
        const promises = [];
        promises.push(lazyLoadLazyImgTags(el));
        promises.push(lazyLoadLazyImgComponents(el));
        await Promise.all(promises);
        resolve();
    });
}
function lazyLoadLazyImgTags(el) {
    return new Promise(async (resolve) => {
        const images = getAllElements(el, 'img');
        await lazyLoadSelectedImages(images);
        resolve();
    });
}
function lazyLoadLazyImgComponents(el) {
    return new Promise(async (resolve) => {
        const images = getAllElements(el, 'deckgo-lazy-img');
        await lazyLoadSelectedLazyImagesComponent(images);
        resolve();
    });
}
function hideLazyLoadImages(el) {
    return new Promise((resolve) => {
        let images = getAllElements(el, 'img');
        if (!images) {
            resolve();
        }
        else {
            images = images.filter((image) => image.getAttribute('data-src'));
            images.forEach((image) => {
                image.style.setProperty('visibility', 'hidden');
            });
            resolve();
        }
    });
}

function showAllRevealElements(el) {
    return new Promise(async (resolve) => {
        const elements = el.querySelectorAll('deckgo-reveal, deckgo-reveal-list');
        if (elements && elements.length > 0) {
            const promises = [];
            for (const element of Array.from(elements)) {
                promises.push(element.revealAll());
            }
            await Promise.all(promises);
        }
        resolve();
    });
}
function hideAllRevealElements(el) {
    return new Promise(async (resolve) => {
        const elements = el.querySelectorAll('deckgo-reveal, deckgo-reveal-list');
        if (elements && elements.length > 0) {
            const promises = [];
            for (const element of Array.from(elements)) {
                promises.push(element.hideAll());
            }
            await Promise.all(promises);
        }
        resolve();
    });
}
function afterSwipe() {
    return new Promise((resolve) => {
        resolve();
    });
}
function lazyLoadContent(el) {
    return new Promise(async (resolve) => {
        const promises = [];
        promises.push(lazyLoadImages(el));
        promises.push(lazyLoadComponentContent(el, 'deckgo-gif'));
        promises.push(lazyLoadComponentContent(el, 'deckgo-youtube'));
        promises.push(lazyLoadComponentContent(el, 'deckgo-demo'));
        promises.push(lazyLoadComponentContent(el, 'deckgo-word-cloud'));
        await Promise.all(promises);
        resolve();
    });
}

const deckdeckgoSlideBigImgCss = ":host{display:block;position:relative;background:var(--background);color:var(--color);height:inherit;z-index:var(--zIndex, 1);--slide-padding-top-default:64px;--slide-padding-end-default:64px;--slide-padding-bottom-default:64px;--slide-padding-start-default:64px}@media screen and (max-width: 1024px){:host{--slide-padding-top-default:32px;--slide-padding-end-default:32px;--slide-padding-bottom-default:32px;--slide-padding-start-default:32px}}div.deckgo-slide{display:flex;overflow:var(--overflow, hidden);padding:var(--slide-padding-top, var(--slide-padding-top-default)) var(--slide-padding-end, var(--slide-padding-end-default)) var(--slide-padding-bottom, var(--slide-padding-bottom-default)) var(--slide-padding-start, var(--slide-padding-start-default));width:calc(var(--slide-width) - var(--slide-padding-start, var(--slide-padding-start-default)) - var(--slide-padding-end, var(--slide-padding-end-default)));height:calc( var(--slide-height) - var(--slide-padding-top, var(--slide-padding-top-default)) - var(--slide-padding-bottom, var(--slide-padding-bottom-default)) );min-height:calc( var(--slide-min-height) - var(--slide-padding-top, var(--slide-padding-top-default)) - var(--slide-padding-bottom, var(--slide-padding-bottom-default)) );-webkit-user-select:var(--slide-user-select, none);-moz-user-select:var(--slide-user-select, none);-ms-user-select:var(--slide-user-select, none);user-select:var(--slide-user-select, none);user-drag:none;-webkit-user-drag:none}@media print{div.deckgo-slide{padding:0;page-break-after:always;height:100vh}::slotted([slot=actions]){display:none}::slotted([slot=background]){display:var(--slide-background-print-display, none)}}::slotted(ul),::slotted(ol){-webkit-padding-start:var(--slide-padding-start, var(--slide-padding-start-default));padding-inline-start:var(--slide-padding-start, var(--slide-padding-start-default))}::slotted([slot=notes]){display:none}::slotted([slot=actions]){position:absolute;top:var(--slide-actions-top, 16px);right:var(--slide-actions-end, 32px);left:var(--slide-actions-start);display:var(--slide-actions-display);z-index:var(--slide-actions-z-index)}::slotted([slot=background]){position:var(--slide-background-position, absolute);top:var(--slide-background-top, 0);right:var(--slide-background-end);left:var(--slide-background-start, 0);width:var(--slide-background-width);height:var(--slide-background-height);z-index:-2;pointer-events:none}::slotted([slot=header]),::slotted([slot=footer]){position:absolute;left:0;width:var(--slide-width);height:100%;display:flex;align-items:center;z-index:var(--slide-header-footer-z-index, -1)}::slotted([slot=header]){top:0;width:calc(var(--slide-width) - var(--slide-header-margin-start, 32px) - var(--slide-header-margin-end, 32px));max-height:var(--slide-header-max-height, 48px);justify-content:var(--slide-header-justify-content, flex-start);margin:var(--slide-header-margin-top, 16px) var(--slide-header-margin-end, 32px) var(--slide-header-margin-bottom, 16px) var(--slide-header-margin-start, 32px)}@media screen and (max-width: 1024px){::slotted([slot=header]){max-height:var(--slide-header-max-height, 16px)}}::slotted([slot=footer]){bottom:0;width:calc(var(--slide-width) - var(--slide-footer-margin-start, 16px) - var(--slide-footer-margin-end, 16px));max-height:var(--slide-footer-max-height, 32px);justify-content:var(--slide-footer-justify-content, center);margin:var(--slide-footer-margin-top, 16px) var(--slide-footer-margin-end, 16px) var(--slide-footer-margin-bottom, 16px) var(--slide-footer-margin-start, 16px)}div.deckgo-slide{flex-direction:column;justify-content:center;align-items:center;margin:0 0}div.deckgo-slide .deckgo-big-img-container{overflow:hidden;text-align:center}div.deckgo-slide .deckgo-big-img-container img{max-width:var(--slide-img-max-width)}div.deckgo-slide .deckgo-big-img-container img:not(.cropped){max-width:100%;max-height:100%}::slotted([slot=title]){display:none}";

const capitalize = (str) => str.charAt(0).toUpperCase() + str.slice(1);
const DeckdeckgoSlideBigImg = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.slideDidLoad = createEvent(this, "slideDidLoad", 7);
    this.imgSrc = '';
    this.imgDivisions = '';
    this.axis = 'x';
    this.reverse = false;
    this.currentStep = -1;
  }
  get divisions() {
    return this.imgDivisions.split(';').map((str) => {
      const num = parseInt(str);
      if (isNaN(num)) {
        return 0;
      }
      return num;
    });
  }
  async componentDidLoad() {
    await hideLazyLoadImages(this.el);
    this.crop = this.el.shadowRoot.querySelector('.deckgo-big-img-container');
    this.bigImg = this.el.shadowRoot.querySelector('img');
    this.slideDidLoad.emit();
  }
  next() {
    return new Promise((resolve) => {
      this.prevNext(true);
      resolve();
    });
  }
  prev() {
    return new Promise((resolve) => {
      this.prevNext(false);
      resolve();
    });
  }
  prevNext(next) {
    const axisDimension = this.axis === 'x' ? 'width' : 'height';
    const perpendicularAxisDimension = this.axis === 'y' ? 'width' : 'height';
    const axisMarginStart = `margin${this.axis === 'x' ? 'Left' : 'Top'}`;
    if (this.currentStep === -1 && next) {
      this.currentStep = this.reverse ? this.divisions.length : 0;
    }
    else if (this.currentStep === this.divisions.length && !next && this.reverse) {
      this.currentStep = -1;
    }
    else {
      this.currentStep = this.currentStep + (this.reverse ? -1 : 1) * (next ? 1 : -1);
    }
    if (this.currentStep === -1) {
      this.bigImg.classList.remove('cropped');
      this.bigImg.style[axisMarginStart] = '';
      this.bigImg.style[perpendicularAxisDimension] = '';
      this.crop.style[axisDimension] = '';
      this.crop.style[perpendicularAxisDimension] = '';
    }
    else {
      this.crop.style[perpendicularAxisDimension] = '100%';
      this.bigImg.style[perpendicularAxisDimension] = '100%';
      this.bigImg.classList.add('cropped');
      const previousNaturalDivision = this.currentStep === 0 ? 0 : this.divisions[this.currentStep - 1];
      const calcCrop = () => {
        const imgClientLength = this.bigImg[`client${capitalize(axisDimension)}`];
        const imgNaturalLength = this.bigImg[`natural${capitalize(axisDimension)}`];
        const lengthFactor = imgClientLength / imgNaturalLength;
        const currentNaturalDivision = this.currentStep === this.divisions.length ? imgNaturalLength : this.divisions[this.currentStep];
        return {
          length: (currentNaturalDivision - previousNaturalDivision) * lengthFactor,
          shift: -(previousNaturalDivision * lengthFactor),
        };
      };
      let crop = calcCrop();
      if (crop.length > this.el.shadowRoot.querySelector('.deckgo-big-img-container').clientHeight) {
        this.crop.style[perpendicularAxisDimension] = '';
        crop = calcCrop();
      }
      this.crop.style[axisDimension] = crop.length + 'px';
      this.bigImg.style[axisMarginStart] = crop.shift + 'px';
    }
  }
  isEnd() {
    return this.reverse ? this.currentStep === 0 : this.currentStep === this.divisions.length;
  }
  isBeginning() {
    return this.currentStep === -1;
  }
  beforeSwipe(enter) {
    return new Promise(async (resolve) => {
      const couldSwipe = !this.divisions[0] || (enter ? this.isEnd() : this.isBeginning());
      if (couldSwipe) {
        resolve(true);
        return;
      }
      if (enter) {
        await this.next();
      }
      else {
        await this.prev();
      }
      resolve(false);
    });
  }
  afterSwipe() {
    return afterSwipe();
  }
  lazyLoadContent() {
    return lazyLoadContent(this.el);
  }
  revealContent() {
    return showAllRevealElements(this.el);
  }
  hideContent() {
    return hideAllRevealElements(this.el);
  }
  render() {
    return (h(Host, { class: { 'deckgo-slide-container': true } }, h("div", { class: "deckgo-slide" }, h("slot", { name: "title" }), h("div", { class: "deckgo-big-img-container" }, h("img", { "data-src": this.imgSrc, alt: this.imgAlt }), h("slot", { name: "notes" }), h("slot", { name: "actions" }), h("slot", { name: "background" }), h("slot", { name: "header" }), h("slot", { name: "footer" })))));
  }
  get el() { return getElement(this); }
};
DeckdeckgoSlideBigImg.style = deckdeckgoSlideBigImgCss;

export { DeckdeckgoSlideBigImg as deckgo_slide_big_img };
